@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -35,12 +35,15 @@ module like it's done in "isStandardLibraryPath" of this module.
 
 """
 
-from . import Options, Utils
-
-import sys, os, imp
+from __future__ import print_function
 
+import imp
+import os
+import sys
 from logging import warning
 
+from . import Options, Utils
+
 _debug_module_finding = False
 
 warned_about = set()
@@ -48,17 +51,20 @@ warned_about = set()
 # Directory where the main script lives. Should attempt to import from there.
 main_path = None
 
-def setMainScriptDirectory( main_dir ):
+def setMainScriptDirectory(main_dir):
     # We need to set this from the outside, pylint: disable=W0603
 
     global main_path
     main_path = main_dir
 
-def isPackageDir( dirname ):
+def isPackageDir(dirname):
     return Utils.isDir( dirname ) and \
-           Utils.isFile( Utils.joinpath( dirname, "__init__.py" ))
+           (
+               Utils.python_version >= 330 or
+               Utils.isFile( Utils.joinpath( dirname, "__init__.py" ) )
+           )
 
-def findModule( source_ref, module_name, parent_package, level, warn ):
+def findModule(source_ref, module_name, parent_package, level, warn):
     # We have many branches here, because there are a lot of cases to try.
     # pylint: disable=R0912
 
@@ -86,7 +92,7 @@ def findModule( source_ref, module_name, parent_package, level, warn ):
                     elif level == -1:
                         level_desc = "as relative or absolute import"
                     elif level == 1:
-                        level_desc = "one package level up" % level
+                        level_desc = "%d package level up" % level
                     else:
                         level_desc = "%d package levels up" % level
 
@@ -122,48 +128,98 @@ def findModule( source_ref, module_name, parent_package, level, warn ):
         module_filename = None
 
     if _debug_module_finding:
-        print( "findModule: Result", module_package_name, module_name, module_filename )
+        print(
+            "findModule: Result",
+            module_package_name,
+            module_name,
+            module_filename
+        )
 
     return module_package_name, module_name, module_filename
 
-def _findModuleInPath( module_name, package_name ):
+def _impFindModuleWrapper(module_name, search_path):
+    """ This wraps imp.find_module because Python3.3 bugs.
+
+    Python3.3 accepts imports on directory names in PYTHONPATH, but does not
+    return them from imp.find_module, which it also deprecated, but would be
+    asking us to use the many variants in importlib manually. So this only
+    fixes up the one issue it has, that it won't accept these namespace dirs.
+
+    TODO: That probably is not sufficient to cover actual namespace packages,
+    where multiple such directories are to be logically joined.
+    """
+    try:
+        # Does not accept keyword arguments, another thing this wrapper gives
+        # us then.
+        module_fh, module_filename, _module_desc = imp.find_module(
+            module_name,
+            search_path
+        )
+    except ImportError:
+        if Utils.python_version >= 330:
+            for path_element in search_path:
+                candidate = Utils.joinpath( path_element, module_name )
+
+                if Utils.isDir( candidate ):
+                    module_filename = candidate
+                    module_fh = None
+
+                    break
+            else:
+                raise
+        else:
+            raise
+
+    # Close the file handle, we won't use it.
+    if module_fh is not None:
+        module_fh.close()
+
+    return module_filename
+
+
+def _findModuleInPath(module_name, package_name):
     # We have many branches here, because there are a lot of cases to try.
     # pylint: disable=R0912
 
     if _debug_module_finding:
-        print( "_findModuleInPath: Enter", module_name, package_name )
+        print("_findModuleInPath: Enter", module_name, "in", package_name)
 
     assert main_path is not None
-    extra_paths = [ os.getcwd(), main_path  ]
+    extra_paths = [os.getcwd(), main_path]
 
     if package_name is not None:
         # Work around imp.find_module bug on at least Windows. Won't handle
         # module name empty in find_module. And thinking of it, how could it
         # anyway.
         if module_name == "":
-            module_name = package_name.split( "." )[ -1 ]
-            package_name = ".".join( package_name.split( "." )[:-1] )
+            module_name = package_name.split(".")[-1]
+            package_name = ".".join(package_name.split(".")[:-1])
 
-        def getPackageDirname( element ):
-            return Utils.joinpath( element, *package_name.split( "." ) )
+        def getPackageDirname(element):
+            return Utils.joinpath(element,*package_name.split("."))
 
         ext_path = [
-            getPackageDirname( element )
+            getPackageDirname(element)
             for element in
             extra_paths + sys.path
-            if isPackageDir( getPackageDirname( element ) )
+            if isPackageDir(getPackageDirname(element))
         ]
 
         if _debug_module_finding:
-            print( "_findModuleInPath: Package, using extended path", ext_path )
+            print("_findModuleInPath: Package, using extended path", ext_path)
 
         try:
-            module_fh, module_filename, _module_desc = imp.find_module( module_name, ext_path )
-            if module_fh is not None:
-                module_fh.close()
+            module_filename = _impFindModuleWrapper(
+                module_name = module_name,
+                search_path = ext_path
+            )
 
             if _debug_module_finding:
-                print( "_findModuleInPath: imp.find_module worked", module_filename, package_name )
+                print(
+                    "_findModuleInPath: imp.find_module worked",
+                    module_filename,
+                    package_name
+                )
 
             return module_filename, package_name
         except ImportError:
@@ -172,42 +228,40 @@ def _findModuleInPath( module_name, package_name ):
         except SyntaxError:
             # Warn user, as this is kind of unusual.
             warning(
-                "%s: Module cannot be imported due to syntax errors",
+                "%s: Module cannot be imported due to syntax errors.",
                 module_name,
             )
 
-            if _debug_module_finding:
-                print( "_findModuleInPath: imp.find_module failed with syntax error" )
+            return None, None
 
     ext_path = extra_paths + sys.path
 
     if _debug_module_finding:
-        print( "_findModuleInPath: Non-package, using extended path", ext_path )
+        print("_findModuleInPath: Non-package, using extended path", ext_path)
 
     try:
-        module_fh, module_filename, _module_desc = imp.find_module( module_name, ext_path )
-        if module_fh is not None:
-            module_fh.close()
+        module_filename = _impFindModuleWrapper(
+            module_name = module_name,
+            search_path = ext_path
+        )
     except SyntaxError:
         # Warn user, as this is kind of unusual.
         warning(
-            "%s: Module cannot be imported due to syntax errors",
+            "%s: Module cannot be imported due to syntax errors.",
             module_name,
         )
 
-        if _debug_module_finding:
-            print( "_findModuleInPath: imp.find_module failed with syntax error" )
-
-        module_filename = None
+        return None, None
 
     if _debug_module_finding:
-        print( "_findModuleInPath: imp.find_module gave", module_filename )
+        print("_findModuleInPath: imp.find_module gave", module_filename)
 
     return module_filename, None
 
-def _findModule( module_name, parent_package ):
+
+def _findModule(module_name, parent_package):
     if _debug_module_finding:
-        print( "_findModule: Enter", module_name, "in", parent_package )
+        print("_findModule: Enter", module_name, "in", parent_package)
 
     # The os.path is strangely hacked into the os module, dispatching per
     # platform, we either cannot look into it, or we require that we resolve it
@@ -215,16 +269,19 @@ def _findModule( module_name, parent_package ):
     if module_name == "os.path" and parent_package is None:
         parent_package = "os"
 
-        if not Options.isWindowsTarget():
-            module_name = Utils.basename( os.path.__file__ ).replace( ".pyc", "" )
-        else:
-            module_name = "ntpath"
+        module_name = Utils.basename(os.path.__file__)
+        if module_name.endswith(".pyc"):
+            module_name = module_name[:-4]
 
     assert module_name != "" or parent_package is not None
 
+    # Built-in module names must not be searched any further.
+    if module_name in sys.builtin_module_names and parent_package is None:
+        return None, None
+
     if "." in module_name:
-        package_part = module_name[ : module_name.rfind( "." ) ]
-        module_name = module_name[ module_name.rfind( "." ) + 1 : ]
+        package_part = module_name[ : module_name.rfind(".") ]
+        module_name = module_name[ module_name.rfind(".") + 1 : ]
 
         # Relative import
         if parent_package is not None:
@@ -252,7 +309,7 @@ def _findModule( module_name, parent_package ):
 
         return module_filename, package
 
-def _isWhiteListedNotExistingModule( module_name ):
+def _isWhiteListedNotExistingModule(module_name):
     white_list = (
         "mac", "nt", "os2", "posix", "_emx_link", "riscos", "ce", "riscospath",
         "riscosenviron", "Carbon.File", "org.python.core", "_sha", "_sha256",
@@ -362,6 +419,20 @@ areallylongpackageandmodulenametotestreprtruncation""",
 
         # Very old modules with older names
         "simplejson", "sets",
+
+        # Standalone mode "site" import flexibilities
+        "sitecustomize", "usercustomize", "apport_python_hook",
+        "_frozen_importlib",
+
+        # Standard library stuff that is optional
+        "comtypes.server.inprocserver", "_tkinter", "_scproxy", "EasyDialogs",
+        "SOCKS", "rourl2path", "_winapi", "win32api", "win32con", "_gestalt",
+        "java.lang", "vms_lib", "ic", "readline", "termios", "_sysconfigdata",
+        "al", "AL", "sunaudiodev", "SUNAUDIODEV", "Audio_mac",
+        "test.test_MimeWriter", "dos", "win32pipe", "Carbon", "Carbon.Files",
+        "sgi", "ctypes.macholib.dyld", "bsddb3", "_pybsddb", "_xmlrpclib",
+        "netbios", "win32wnet", "email.Parser", "elementree.cElementTree",
+        "elementree.ElementTree", "_gbdm",
     )
 
     # TODO: Turn this into a warning that encourages reporting.
@@ -371,14 +442,73 @@ areallylongpackageandmodulenametotestreprtruncation""",
 
     return module_name in white_list
 
-def isStandardLibraryPath( path ):
-    path = Utils.normcase( path )
-    os_path = Utils.normcase( Utils.dirname( os.__file__  ) )
 
-    if not path.startswith( os_path ):
-        return False
+def getStandardLibraryPaths():
 
+    # Using the function object to cache its result, avoiding global variable
+    # usage.
+    if not hasattr(getStandardLibraryPaths, "result"):
+        os_filename = os.__file__
+        if os_filename.endswith(".pyc"):
+            os_filename = os_filename[:-1]
+
+        os_path = Utils.normcase(Utils.dirname(os_filename))
+
+        stdlib_paths = set([os_path])
+
+        # Happens for virtualenv situation, some modules will come from the link
+        # this points to.
+        if Utils.isLink(os_filename):
+            os_filename = Utils.readLink(os_filename)
+            stdlib_paths.add(Utils.normcase(Utils.dirname(os_filename)))
+
+        # Another possibility is "orig-prefix.txt" file near the os.py, which
+        # points to the original install.
+        orig_prefix_filename = Utils.joinpath(os_path, "orig-prefix.txt")
+
+        if Utils.isFile(orig_prefix_filename):
+            stdlib_paths.add(
+                Utils.normcase(
+                    Utils.joinpath(
+                        open(orig_prefix_filename).read(),
+                        "lib"
+                    )
+                )
+            )
+
+        # And yet another possibility, for MacOS Homebrew created virtualenv
+        # at least is a link ".Python", which points to the original install.
+        python_link_filename = Utils.joinpath(os_path, "..", ".Python")
+        if Utils.isLink(python_link_filename):
+            stdlib_paths.add(
+                Utils.normcase(
+                    Utils.joinpath(
+                        Utils.readLink(python_link_filename),
+                        "lib"
+                    )
+                )
+            )
+
+        getStandardLibraryPaths.result = stdlib_paths
+
+    return getStandardLibraryPaths.result
+
+
+def isStandardLibraryPath(path):
+    path = Utils.normcase(path)
+
+    # In virtualenv, the "site.py" lives in a place that suggests it is not in
+    # standard library, although it is.
+    if os.path.basename(path) == "site.py":
+        return True
+
+    # These never are in standard library paths.
     if "dist-packages" in path or "site-packages" in path:
         return False
 
-    return True
+
+    for candidate in getStandardLibraryPaths():
+        if path.startswith(candidate):
+            return True
+    else:
+        return False
