@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -18,24 +18,25 @@
 """ Options module """
 
 version_string = """\
-Nuitka V0.4.5
-Copyright (C) 2013 Kay Hayen."""
+Nuitka V0.5.3.3
+Copyright (C) 2014 Kay Hayen."""
+
+import logging
+import re
+import sys
+from optparse import SUPPRESS_HELP, OptionGroup, OptionParser
 
 from . import Utils
 
-from optparse import OptionParser, OptionGroup
-
-import sys, logging
-
-# Indicator if we were called as "nuitka-python" in which case we assume some
+# Indicator if we were called as "nuitka-run" in which case we assume some
 # other defaults and work a bit different with parameters.
-is_nuitka_python = Utils.basename( sys.argv[0] ).lower().startswith( "nuitka-python" )
+is_nuitka_run = Utils.basename(sys.argv[0]).lower().startswith("nuitka-run")
 
 def getVersion():
     return version_string.split()[1][1:]
 
-if not is_nuitka_python:
-    usage = "usage: %prog [--exe] [--execute] [options] main_module.py"
+if not is_nuitka_run:
+    usage = "usage: %prog [--module] [--execute] [options] main_module.py"
 else:
     usage = "usage: %prog [options] main_module.py"
 
@@ -44,19 +45,49 @@ parser = OptionParser(
     version = getVersion()
 )
 
+# This option is obsolete, and module should be used.
 parser.add_option(
     "--exe",
     action  = "store_true",
-    dest    = "executable",
-    default = is_nuitka_python,
-    help    = """\
-Create a standalone executable instead of a compiled extension module. Default is %s.""" %
-       ( "on" if is_nuitka_python else "off" )
+    dest    = "obsolete_executable",
+    default = False,
+    help    = SUPPRESS_HELP
 )
 
+parser.add_option(
+    "--module",
+    action  = "store_false",
+    dest    = "executable",
+    default = True,
+    help    = """\
+Create an extension module executable instead of a program. Defaults to off."""
+)
+
+parser.add_option(
+    "--standalone", "--portable",
+    action  = "store_true",
+    dest    = "is_standalone",
+    default = False,
+    help    = """\
+Enable standalone mode in build. This allows you to transfer the created binary
+to other machines without it relying on an existing Python installation. It
+implies these options: "--recurse-all --recurse-stdlib". Defaults to off.""",
+)
+
+parser.add_option(
+    "--nofreeze-stdlib",
+    action  = "store_false",
+    dest    = "freeze_stdlib",
+    default = True,
+    help    = """\
+In standalone mode by default all modules of standard library will be frozen
+as bytecode. As a result compilation time will increase very much.
+""",
+    )
+
 recurse_group = OptionGroup(
     parser,
-    "Control the recursion into imported modules with '--exe' mode"
+    "Control the recursion into imported modules"
 )
 
 
@@ -65,7 +96,8 @@ recurse_group.add_option(
     action  = "store_true",
     dest    = "recurse_stdlib",
     default = False,
-    help    = "Also descend into imported modules from standard library."
+    help    = """\
+Also descend into imported modules from standard library. Defaults to off."""
 )
 
 recurse_group.add_option(
@@ -74,8 +106,8 @@ recurse_group.add_option(
     dest    = "recurse_none",
     default = False,
     help    = """\
-When --recurse-none is used, do not descend into any imported modules at all, overrides
-all other recursion options. Default %default."""
+When --recurse-none is used, do not descend into any imported modules at all,
+overrides all other recursion options. Defaults to off."""
 )
 
 recurse_group.add_option(
@@ -85,7 +117,7 @@ recurse_group.add_option(
     default = False,
     help    = """\
 When --recurse-all is used, attempt to descend into all imported modules.
-Default %default."""
+Defaults to off."""
 )
 
 recurse_group.add_option(
@@ -95,8 +127,8 @@ recurse_group.add_option(
     metavar = "MODULE/PACKAGE",
     default = [],
     help    = """\
-Recurse to that module, or if a package, to the whole package. Can be given multiple
-times. Default empty."""
+Recurse to that module, or if a package, to the whole package. Can be given
+multiple times. Default empty."""
 )
 
 recurse_group.add_option(
@@ -106,8 +138,9 @@ recurse_group.add_option(
     metavar = "MODULE/PACKAGE",
     default = [],
     help    = """\
-Do not recurse to that module, or if a package, to the whole package in any case,
-overrides all other options. Can be given multiple times. Default empty."""
+Do not recurse to that module, or if a package, to the whole package in any
+case, overrides all other options. Can be given multiple times. Default
+empty."""
 )
 
 recurse_group.add_option(
@@ -117,8 +150,9 @@ recurse_group.add_option(
     metavar = "MODULE/PACKAGE",
     default = [],
     help    = """\
-Recurse into that directory, no matter if it's used by the given main program in a
-visible form. Overrides all other options. Can be given multiple times. Default empty."""
+Recurse into that directory, no matter if it's used by the given main program
+in a visible form. Overrides all other options. Can be given multiple times.
+Default empty."""
 )
 
 parser.add_option_group( recurse_group )
@@ -129,14 +163,14 @@ execute_group = OptionGroup(
 )
 
 execute_group.add_option(
-    "--execute",
+    "--run", "--execute",
     action  = "store_true",
     dest    = "immediate_execution",
-    default = is_nuitka_python,
+    default = is_nuitka_run,
     help    = """\
-Execute immediately the created binary (or import the compiled module). Default
-is %s.""" %
-       ( "on" if is_nuitka_python else "off" )
+Execute immediately the created binary (or import the compiled module).
+Defaults to %s.""" %
+       ("on" if is_nuitka_run else "off")
 )
 
 execute_group.add_option(
@@ -145,8 +179,9 @@ execute_group.add_option(
     dest    = "keep_pythonpath",
     default = False,
     help    = """\
-When immediately executing the created binary (--execute), don't reset PYTHONPATH. When
-all modules are successfully included, you ought to not need PYTHONPATH anymore."""
+When immediately executing the created binary (--execute), don't reset
+PYTHONPATH. When all modules are successfully included, you ought to not need
+PYTHONPATH anymore."""
 )
 
 parser.add_option_group( execute_group )
@@ -157,42 +192,52 @@ dump_group = OptionGroup(
 )
 
 dump_group.add_option(
-    "--dump-xml",
+    "--dump-xml", "--xml",
     action  = "store_true",
     dest    = "dump_xml",
     default = False,
     help    = """Dump the final result of optimization as XML, then exit."""
 )
 
-dump_group.add_option(
-    "--dump-tree",
-    action  = "store_true",
-    dest    = "dump_tree",
-    default = False,
-    help    = """Dump the final result of optimization as text, then exit."""
-)
-
 dump_group.add_option(
     "--display-tree",
     action  = "store_true",
     dest    = "display_tree",
     default = False,
-    help    = """Display the final result of optimization in a GUI, then exit."""
+    help    = """\
+Display the final result of optimization in a GUI, then exit."""
 )
 
 parser.add_option_group( dump_group )
 
+def getSupportedPythonVersions():
+    return ("2.6", "2.7", "3.2", "3.3")
+
+def getSupportedPythonVersionStr():
+    supported_python_versions = getSupportedPythonVersions()
+
+    supported_python_versions_str = repr(supported_python_versions)[1:-1]
+    supported_python_versions_str = re.sub(
+        r"(.*),(.*)$",
+        r"\1, or\2",
+        supported_python_versions_str
+    )
+
+    return supported_python_versions_str
+
 parser.add_option(
     "--python-version",
     action  = "store",
     dest    = "python_version",
-    choices = ( "2.6", "2.7", "3.2", "3.3" ),
+    choices = getSupportedPythonVersions(),
     default = None,
-    help    = """Major version of Python to be used, one of '2.6', '2.7', or '3.2'."""
+    help    = """Major version of Python to be used, one of %s.""" % (
+       getSupportedPythonVersionStr()
+    )
 )
 
 parser.add_option(
-    "--python-debug",
+    "--python-debug", "--python-dbg",
     action  = "store_true",
     dest    = "python_debug",
     default = None,
@@ -201,19 +246,42 @@ Use debug version or not. Default uses what you are using to run Nuitka, most
 likely a non-debug version."""
 )
 
+parser.add_option(
+    "--python-flag",
+    action  = "append",
+    dest    = "python_flags",
+    default = [],
+    help    = """\
+Python flags to use. Default uses what you are using to run Nuitka, this
+enforces a specific mode. These are options that also exist to standard
+Python executable. Currently supported "-S" (alias nosite) ,
+"static_hashes" (not use Randomization). Default empty."""
+)
+
 codegen_group = OptionGroup(
     parser,
     "Code generation choices"
 )
 
+codegen_group.add_option(
+    "--improved", "--enhanced",
+    action  = "store_true",
+    dest    = "improved",
+    default = False,
+    help    = """\
+Allow minor devitations from CPython behaviour, e.g. better tracebacks, which
+are not really incompatible, but different.""",
+)
+
+
 codegen_group.add_option(
     "--code-gen-no-statement-lines",
     action  ="store_false",
     dest    = "statement_lines",
     default = True,
     help    = """\
-Statements shall have their line numbers set. Disable this for less precise exceptions and
-slightly faster code. Not recommended. Defaults to off."""
+Statements shall have their line numbers set. Disable this for less precise
+exceptions and slightly faster code. Not recommended. Defaults to off."""
 )
 
 codegen_group.add_option(
@@ -221,7 +289,8 @@ codegen_group.add_option(
     action  = "store_true",
     dest    = "no_optimize",
     default = False,
-    help    = """Disable all unnecessary optimizations on Python level. Defaults to off."""
+    help    = SUPPRESS_HELP
+# """Disable all unnecessary optimizations on Python level. Defaults to off."""
 )
 
 parser.add_option_group( codegen_group )
@@ -239,7 +308,8 @@ outputdir_group.add_option(
     default = "",
     help    = """\
 Specify where intermediate and final output files should be put. DIRECTORY will
-be populated with C++ files, object files, etc. Defaults to current directory."""
+be populated with C++ files, object files, etc. Defaults to current directory.
+"""
 )
 
 outputdir_group.add_option(
@@ -249,20 +319,11 @@ outputdir_group.add_option(
     default = False,
     help    = """\
 Removes the build directory after producing the module or exe file.
-Default %default."""
+Defaults to off."""
 )
 
 parser.add_option_group( outputdir_group )
 
-parser.add_option(
-    "--windows-target",
-    action  = "store_true",
-    dest    = "windows_target",
-    default = False,
-    help    = """\
-Force compilation for windows, useful for cross-compilation. Defaults to off."""
-)
-
 parser.add_option(
     "--windows-disable-console",
     action  = "store_true",
@@ -314,8 +375,9 @@ debug_group.add_option(
     dest    = "cpp_only",
     default = False,
     help    = """\
-Compile the would-be regenerated source file. Allows compiling edited C++ files with the
-C++ compiler for quick debugging changes to the generated source. Defaults to off."""
+Compile the would-be regenerated source file. Allows compiling edited C++ files
+with the C++ compiler for quick debugging changes to the generated source.
+Defaults to off."""
 )
 
 debug_group.add_option(
@@ -324,8 +386,8 @@ debug_group.add_option(
     dest    = "experimental",
     default = False,
     help    = """\
-Use features declared as 'experimental'. May have no effect if no experimental features
-are present in the code. Defaults to off."""
+Use features declared as 'experimental'. May have no effect if no experimental
+features are present in the code. Defaults to off."""
 )
 
 parser.add_option_group( debug_group )
@@ -350,6 +412,16 @@ Enforce the use of clang (clang 3.0 or higher).
 Defaults to off."""
 )
 
+parser.add_option(
+    "--mingw",
+    action  = "store_true",
+    dest    = "mingw",
+    default = False,
+    help    = """\
+Enforce the use of MinGW on Windows.
+Defaults to off."""
+)
+
 tracing_group = OptionGroup(
     parser,
     "Tracing features"
@@ -361,7 +433,8 @@ tracing_group.add_option(
     dest    = "show_scons",
     default = False,
     help    = """\
-Operate Scons in non-quiet mode, showing the executed commands. Defaults to off."""
+Operate Scons in non-quiet mode, showing the executed commands.
+Defaults to off."""
 )
 
 tracing_group.add_option(
@@ -369,7 +442,17 @@ tracing_group.add_option(
     action  = "store_true",
     dest    = "show_progress",
     default = False,
-    help    = """Provide progress information and statistics. Defaults to off."""
+    help    = """Provide progress information and statistics.
+Defaults to off."""
+)
+
+tracing_group.add_option(
+    "--show-modules",
+    action  = "store_true",
+    dest    = "show_inclusion",
+    default = False,
+    help    = """Provide a final summary on included modules.
+Defaults to off."""
 )
 
 tracing_group.add_option(
@@ -378,7 +461,8 @@ tracing_group.add_option(
     dest    = "verbose",
     default = False,
     help    = """\
-Output details of actions take, esp. in optimizations. Can become a lot."""
+Output details of actions take, esp. in optimizations. Can become a lot.
+Defaults to off."""
 )
 
 
@@ -391,19 +475,10 @@ parser.add_option(
     metavar = "N",
     default = Utils.getCoreCount(),
     help    = """\
-Specify the allowed number of parallel C++ compiler jobs. Defaults to the system
-CPU count.""",
+Specify the allowed number of parallel C++ compiler jobs. Defaults to the
+system CPU count.""",
 )
 
-parser.add_option(
-    "--improved",
-    action  = "store_true",
-    dest    = "improved",
-    default = False,
-    help    = """\
-Allow minor devitations from CPython behaviour, e.g. better tracebacks, which are not
-really incompatible, but different.""",
-)
 
 parser.add_option(
     "--warn-implicit-exceptions",
@@ -414,17 +489,19 @@ parser.add_option(
 Given warnings for implicit exceptions detected at compile time.""",
 )
 
-if False:
-    parser.add_option(
-        "--portable",
-        action  = "store_true",
-        dest    = "is_portable_mode",
-        default = False,
-        help    = """\
-Enable portable mode in build.""",
-    )
 
-if is_nuitka_python:
+parser.add_option(
+    "--icon",
+    action  = "store",
+    dest    = "icon_path",
+    metavar = "ICON_PATH",
+    default = None,
+    help    = """Add executable icon (windows only).""",
+)
+
+# First, isolate the first non-option arguments. TODO: Should repect "--"
+# as a terminator to options.
+if is_nuitka_run:
     count = 0
 
     for count, arg in enumerate( sys.argv ):
@@ -440,7 +517,6 @@ if is_nuitka_python:
 else:
     extra_args = []
 
-
 options, positional_args = parser.parse_args()
 
 if not positional_args:
@@ -451,6 +527,16 @@ Error, need positional argument with python module or main program.""" )
 
 if options.verbose:
     logging.getLogger().setLevel( logging.DEBUG )
+else:
+    logging.getLogger().setLevel( logging.INFO )
+
+# Standalone mode implies an executable, not importing "site" module, which is
+# only for this machine, recursing to all modules, and even including the
+# standard library.
+if options.is_standalone:
+    options.executable = True
+    options.recurse_all = True
+    options.recurse_stdlib = True
 
 def shallTraceExecution():
     return options.trace_execution
@@ -458,16 +544,13 @@ def shallTraceExecution():
 def shallExecuteImmediately():
     return options.immediate_execution
 
-def shallDumpBuiltTree():
-    return options.dump_tree
-
 def shallDumpBuiltTreeXML():
     return options.dump_xml
 
 def shallDisplayBuiltTree():
     return options.display_tree
 
-def shallOnlyExecGcc():
+def shallOnlyExecCppCall():
     return options.cpp_only
 
 def shallHaveStatementLines():
@@ -533,13 +616,16 @@ def shallWarnImplicitRaises():
 def isDebug():
     return options.debug
 
+def isPythonDebug():
+    return options.python_debug or sys.flags.debug
+
 def isOptimize():
     return not options.no_optimize
 
 def isUnstriped():
     return options.unstriped
 
-def getOutputPath( path ):
+def getOutputPath(path):
     if options.output_dir:
         return Utils.normpath( Utils.joinpath( options.output_dir, path ) )
     else:
@@ -572,8 +658,8 @@ def isLto():
 def isClang():
     return options.clang
 
-def isWindowsTarget():
-    return options.windows_target
+def isMingw():
+    return options.mingw
 
 def shallDisableConsoleWindow():
     return options.win_disable_console
@@ -584,6 +670,9 @@ def isFullCompat():
 def isShowProgress():
     return options.show_progress
 
+def isShowInclusion():
+    return options.show_inclusion
+
 def isRemoveBuildDir():
     return options.remove_build
 
@@ -593,7 +682,26 @@ def getIntendedPythonVersion():
 def isExperimental():
     return hasattr( options, "experimental" ) and options.experimental
 
-def isPortableMode():
-    # Temporarily disabled, until it becomes more usable.
-    return False
-    # return options.is_portable_mode
+def isStandaloneMode():
+    return options.is_standalone
+
+def getIconPath():
+    return options.icon_path
+
+def getPythonFlags():
+    result = []
+
+    for part in options.python_flags:
+        if part in ( "-S", "nosite", "no_site" ):
+            result.append( "no_site" )
+        elif part in ( "static_hashes", "norandomization", "no_randomization" ):
+            result.append( "no_randomization" )
+        elif part in ( "-v", "trace_imports", "trace_import" ):
+            result.append( "trace_imports" )
+        else:
+            logging.warning( "Unsupported flag '%s'.", part )
+
+    return result
+
+def freezeAllStdlib():
+    return options.freeze_stdlib
