@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,10 +22,10 @@ of operations allowed.
 
 """
 
-from .Utils import python_version
-
 import operator
 
+from .Utils import python_version
+
 if python_version >= 300:
     operator.div = operator.truediv
     operator.idiv = operator.itruediv
@@ -85,6 +85,31 @@ other_comparison_functions = {
     "NotIn" : lambda value1, value2: value1 not in value2
 }
 
+comparison_inversions = {
+    "Is"    : "IsNot",
+    "IsNot" : "Is",
+    "In"    : "NotIn",
+    "NotIn" : "In"
+}
 
-all_comparison_functions = dict( rich_comparison_functions)
-all_comparison_functions.update( other_comparison_functions )
+all_comparison_functions = dict(rich_comparison_functions)
+all_comparison_functions.update(other_comparison_functions)
+
+def matchException(left, right):
+    if python_version >= 300:
+        if type(right) is tuple:
+            for element in right:
+                if not isinstance(BaseException,element):
+                    raise TypeError("catching classes that do not inherit from BaseException is not allowed")
+        elif not isinstance(BaseException,right):
+            raise TypeError("catching classes that do not inherit from BaseException is not allowed")
+
+    # This doesn't yet work, make it error exit. and silence PyLint for now.
+    # pylint: disable=W0212
+    import os
+    os._exit(16)
+
+    assert False, left
+
+
+all_comparison_functions["exception_match"]=matchException
