@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,20 +22,45 @@
 template_constants_reading = """
 #include "nuitka/prelude.hpp"
 
-// Sentinel PyObject to be used for all our call iterator endings. It will become
-// a PyCObject pointing to NULL. It's address is unique, and that's enough.
+// Sentinel PyObject to be used for all our call iterator endings. It will
+// become a PyCObject pointing to NULL. It's address is unique, and that's
+// enough for us to use it as sentinel value.
 PyObject *_sentinel_value = NULL;
 
 %(constant_declarations)s
 
+#if defined(_WIN32) && defined(_NUITKA_EXE)
+#include <Windows.h>
+const unsigned char* constant_bin;
+struct __initResourceConstants
+{
+    __initResourceConstants()
+    {
+        constant_bin = (const unsigned char*)LockResource(
+            LoadResource(
+                NULL,
+                FindResource(NULL, MAKEINTRESOURCE(3), RT_RCDATA)
+            )
+        );
+    }
+} __initResourceConstants_static_initializer;
+#else
+extern "C" const unsigned char constant_bin[];
+#endif
+
 static void __initConstants( void )
 {
-    if ( %(needs_pickle)s )
-    {
-        UNSTREAM_INIT();
-    }
+    NUITKA_MAY_BE_UNUSED PyObject *exception_type, *exception_value;
+    NUITKA_MAY_BE_UNUSED PyTracebackObject *exception_tb;
+
+#ifdef _MSC_VER
+    // Prevent unused warnings in case of simple programs.
+    (void *)exception_type; (void *)exception_value; (void *)exception_tb;
+#endif
 
 %(constant_inits)s
+
+    return;
 }
 
 void _initConstants( void )
@@ -54,16 +79,3 @@ void _initConstants( void )
     }
 }
 """
-
-template_constants_declaration = """\
-// Call this to initialize all of the below
-void _initConstants( void );
-
-%(constant_declarations)s
-"""
-
-template_reverse_macro = """\
-#define EVAL_ORDERED_%(count)d( %(args)s ) %(expanded)s"""
-
-template_noreverse_macro = """\
-#define EVAL_ORDERED_%(count)d( %(args)s ) %(args)s"""
