@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,13 +20,11 @@
 """
 
 template_function_make_declaration = """\
-#define MAKE_FUNCTION_%(function_identifier)s( %(function_creation_arg_names)s ) _MAKE_FUNCTION_%(function_identifier)s( %(function_creation_arg_reversal)s )
-
-static PyObject *_MAKE_FUNCTION_%(function_identifier)s( %(function_creation_arg_spec)s );
+static PyObject *MAKE_FUNCTION_%(function_identifier)s( %(function_creation_arg_spec)s );
 """
 
 template_function_direct_declaration = """\
-static PyObject *impl_%(function_identifier)s( %(parameter_objects_decl)s );
+%(file_scope)s PyObject *impl_%(function_identifier)s( %(direct_call_arg_spec)s );
 """
 
 function_context_body_template = """
@@ -49,7 +47,7 @@ static void _context_%(function_identifier)s_destructor( void *context_voidptr )
 """
 
 make_function_with_context_template = """
-static PyObject *_MAKE_FUNCTION_%(function_identifier)s( %(function_creation_args)s )
+static PyObject *MAKE_FUNCTION_%(function_identifier)s( %(function_creation_args)s )
 {
     struct _context_%(function_identifier)s_t *_python_context = new _context_%(function_identifier)s_t;
 
@@ -58,10 +56,17 @@ static PyObject *_MAKE_FUNCTION_%(function_identifier)s( %(function_creation_arg
 
     PyObject *result = Nuitka_Function_New(
         %(fparse_function_identifier)s,
-        %(mparse_function_identifier)s,
+        %(dparse_function_identifier)s,
         %(function_name_obj)s,
+#if PYTHON_VERSION >= 330
+        %(function_qualname_obj)s,
+#endif
         %(code_identifier)s,
         %(defaults)s,
+#if PYTHON_VERSION >= 300
+        %(kw_defaults)s,
+        %(annotations)s,
+#endif
         %(module_identifier)s,
         %(function_doc)s,
         _python_context,
@@ -73,14 +78,21 @@ static PyObject *_MAKE_FUNCTION_%(function_identifier)s( %(function_creation_arg
 """
 
 make_function_without_context_template = """
-static PyObject *_MAKE_FUNCTION_%(function_identifier)s( %(function_creation_args)s )
+static PyObject *MAKE_FUNCTION_%(function_identifier)s( %(function_creation_args)s )
 {
     PyObject *result = Nuitka_Function_New(
         %(fparse_function_identifier)s,
-        %(mparse_function_identifier)s,
+        %(dparse_function_identifier)s,
         %(function_name_obj)s,
+#if PYTHON_VERSION >= 330
+        %(function_qualname_obj)s,
+#endif
         %(code_identifier)s,
         %(defaults)s,
+#if PYTHON_VERSION >= 300
+        %(kw_defaults)s,
+        %(annotations)s,
+#endif
         %(module_identifier)s,
         %(function_doc)s
     );
@@ -89,7 +101,7 @@ static PyObject *_MAKE_FUNCTION_%(function_identifier)s( %(function_creation_arg
 }
 """
 
-function_body_template = """\
+template_function_body = """\
 static PyObject *impl_%(function_identifier)s( %(parameter_objects_decl)s )
 {
 %(context_access_function_impl)s
@@ -100,74 +112,55 @@ static PyObject *impl_%(function_identifier)s( %(parameter_objects_decl)s )
     // Actual function code.
 %(function_body)s
 
-    return INCREASE_REFCOUNT( Py_None );
+%(function_exit)s
 }
 """
 
+template_function_exception_exit = """\
+    // Return statement must be present.
+    assert(false);
+function_exception_exit:
+%(function_cleanup)s\
+    assert( exception_type );
+    PyErr_Restore( exception_type, exception_value, (PyObject *)exception_tb );
+    return NULL;
+"""
+
+template_function_noexception_exit = """\
+    // Return statement must be present.
+    assert(false);
+    return NULL;
+"""
+
+template_function_return_exit = """\
+function_return_exit:
+%(function_cleanup)s\
+    return tmp_return_value;
+"""
+
+function_direct_body_template = """\
+%(file_scope)s PyObject *impl_%(function_identifier)s( %(direct_call_arg_spec)s )
+{
+%(context_access_function_impl)s
+
+    // Local variable declarations.
+%(function_locals)s
+
+    // Actual function code.
+%(function_body)s
+
+%(function_exit)s
+}
+"""
+
+
 function_dict_setup = """\
 // Locals dictionary setup.
-PyObjectTemporary locals_dict( PyDict_New() );
+PyObject *locals_dict = PyDict_New();
 """
 
-frame_guard_full_template = """\
-static PyFrameObject *frame_%(frame_identifier)s = NULL;
-
-if ( isFrameUnusable( frame_%(frame_identifier)s ) )
-{
-    if ( frame_%(frame_identifier)s )
-    {
-#if _DEBUG_REFRAME
-        puts( "reframe for %(frame_identifier)s" );
-#endif
-        Py_DECREF( frame_%(frame_identifier)s );
-    }
-
-    frame_%(frame_identifier)s = MAKE_FRAME( %(code_identifier)s, %(module_identifier)s );
-}
-
-FrameGuard frame_guard( frame_%(frame_identifier)s );
-try
-{
-    assert( Py_REFCNT( frame_%(frame_identifier)s ) == 2 ); // Frame stack
-%(codes)s
-}
-catch ( _PythonException &_exception )
-{
-    if ( !_exception.hasTraceback() )
-    {
-        _exception.setTraceback( %(tb_making)s );
-    }
-    else
-    {
-        _exception.addTraceback( frame_guard.getFrame0() );
-    }
-
-    Py_XDECREF( frame_guard.getFrame0()->f_locals );
-    frame_guard.getFrame0()->f_locals = %(frame_locals)s;
-
-    if ( frame_guard.getFrame0() == frame_%(frame_identifier)s )
-    {
-       Py_DECREF( frame_%(frame_identifier)s );
-       frame_%(frame_identifier)s = NULL;
-    }
-%(return_code)s
-}
-catch( ReturnValueException &_exception )
-{
-    return _exception.getValue();
-}"""
-
-frame_guard_python_return = """
-    _exception.toPython();
-    return NULL;"""
-
-frame_guard_cpp_return = """
-    throw;"""
-
-frame_guard_listcontr_template = """\
-FrameGuardVeryLight frame_guard;
-
-%(codes)s"""
+# Bad to read, but the context declaration should be on one line.
+# pylint: disable=C0301
 
 function_context_access_template = """\
     // The context of the function.
