@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,14 +22,11 @@ and its expressions, changing the meaning of course dramatically.
 
 """
 
-from nuitka import Variables, Builtins
-
-from .NodeBases import (
-    ExpressionMixin,
-    NodeBase
-)
+from nuitka import Builtins, Variables
 
 from .ConstantRefNodes import ExpressionConstantRef
+from .NodeBases import ExpressionMixin, NodeBase
+
 
 def _isReadOnlyUnterdeterminedModuleVariable(variable):
     return variable.isModuleVariable() and \
@@ -91,13 +88,8 @@ class ExpressionVariableRef(NodeBase, ExpressionMixin):
     def computeExpression(self, constraint_collection):
         assert self.variable is not None
 
-        if _isReadOnlyUnterdeterminedModuleVariable( self.variable ):
-            constraint_collection.assumeUnclearLocals()
-            constraint_collection.signalChange(
-                "new_expression",
-                self.source_ref,
-                "Unclear module variable delays processing."
-            )
+        if _isReadOnlyUnterdeterminedModuleVariable(self.variable):
+            constraint_collection.assumeUnclearLocals(self.source_ref)
 
         if _isReadOnlyModuleVariable( self.variable ):
             if self.variable_name in Builtins.builtin_exception_names:
@@ -172,6 +164,10 @@ Replaced read-only module attribute '__package__' with constant value."""
         # TODO: Remembered traced could tell better.
         return True
 
+    def mayRaiseException(self, exception_type):
+        # TODO: Remembered traced could tell better.
+        return True
+
 
 class ExpressionTargetVariableRef(ExpressionVariableRef):
     kind = "EXPRESSION_TARGET_VARIABLE_REF"
@@ -258,6 +254,10 @@ class ExpressionTempVariableRef(NodeBase, ExpressionMixin):
     def onContentEscapes(self, constraint_collection):
         constraint_collection.onVariableContentEscapes( self.variable )
 
+    def mayHaveSideEffects(self):
+        # Can't happen
+        return False
+
     def mayRaiseException(self, exception_type):
         # Can't happen
         return False
