@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -15,75 +15,71 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-""" Reformulation of classes
+""" Reformulation of class statements.
 
 Consult the developmer manual for information. TODO: Add ability to sync
 source code comments with developer manual sections.
 
 """
 
-from nuitka.nodes.VariableRefNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTargetVariableRef,
-    ExpressionTempVariableRef,
-    ExpressionVariableRef,
-    StatementTempBlock
+from nuitka import Utils
+from nuitka.nodes.AssignNodes import (
+    StatementAssignmentVariable,
+    StatementDelVariable
 )
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
-from nuitka.nodes.BuiltinRefNodes import ExpressionBuiltinRef
-from nuitka.nodes.ComparisonNodes import ExpressionComparison
-
-from nuitka.nodes.CallNodes import (
-    ExpressionCallNoKeywords,
-    ExpressionCall
-)
-from nuitka.nodes.TypeNodes import ExpressionBuiltinType1
 from nuitka.nodes.AttributeNodes import (
     ExpressionAttributeLookup,
     ExpressionBuiltinHasattr
 )
-from nuitka.nodes.SubscriptNodes import ExpressionSubscriptLookup
-from nuitka.nodes.FunctionNodes import (
-    ExpressionFunctionCreation,
-    ExpressionFunctionBody,
-    ExpressionFunctionCall,
-    ExpressionFunctionRef
-)
+from nuitka.nodes.BuiltinRefNodes import ExpressionBuiltinRef
+from nuitka.nodes.CallNodes import ExpressionCall, ExpressionCallNoKeywords
 from nuitka.nodes.ClassNodes import ExpressionSelectMetaclass
-from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
-from nuitka.nodes.ContainerOperationNodes import (
-    StatementDictOperationRemove,
-    ExpressionDictOperationGet
-)
-from nuitka.nodes.StatementNodes import StatementsSequence
-
+from nuitka.nodes.ComparisonNodes import ExpressionComparison
 from nuitka.nodes.ConditionalNodes import (
     ExpressionConditional,
     StatementConditional
 )
-from nuitka.nodes.ReturnNodes import StatementReturn
-from nuitka.nodes.AssignNodes import StatementAssignmentVariable
-
+from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
+from nuitka.nodes.ContainerOperationNodes import (
+    ExpressionDictOperationGet,
+    StatementDictOperationRemove
+)
+from nuitka.nodes.FunctionNodes import (
+    ExpressionFunctionBody,
+    ExpressionFunctionCall,
+    ExpressionFunctionCreation,
+    ExpressionFunctionRef
+)
 from nuitka.nodes.GlobalsLocalsNodes import (
     ExpressionBuiltinLocals,
     StatementSetLocals
 )
-
 from nuitka.nodes.ParameterSpecs import ParameterSpec
-
-from .Helpers import (
-    makeStatementsSequenceFromStatement,
-    makeSequenceCreationOrConstant,
-    makeDictCreationOrConstant,
-    makeStatementsSequence,
-    buildStatementsNode,
-    extractDocFromBody,
-    buildNodeList,
-    buildNode,
-    getKind
+from nuitka.nodes.ReturnNodes import StatementReturn
+from nuitka.nodes.SubscriptNodes import ExpressionSubscriptLookup
+from nuitka.nodes.TypeNodes import ExpressionBuiltinType1
+from nuitka.nodes.VariableRefNodes import (
+    ExpressionTargetTempVariableRef,
+    ExpressionTargetVariableRef,
+    ExpressionTempVariableRef,
+    ExpressionVariableRef
 )
 
-from nuitka import Utils
+from .Helpers import (
+    buildNode,
+    buildNodeList,
+    buildStatementsNode,
+    extractDocFromBody,
+    getKind,
+    makeDictCreationOrConstant,
+    makeSequenceCreationOrConstant,
+    makeStatementsSequence,
+    makeStatementsSequenceFromStatement,
+    makeTryFinallyStatement,
+    popIndicatorVariable,
+    pushIndicatorVariable
+)
 
 # TODO: Once we start to modify these, we should make sure, the copy is not
 # shared.
@@ -97,23 +93,36 @@ make_class_parameters = ParameterSpec(
 )
 
 
-def _buildClassNode3( provider, node, source_ref ):
+def _buildClassNode3(provider, node, source_ref):
     # Many variables, due to the huge re-formulation that is going on here,
     # which just has the complexity, pylint: disable=R0914
 
     # This function is the Python3 special case with special re-formulation as
     # according to developer manual.
-    class_statements, class_doc = extractDocFromBody( node )
+    class_statements, class_doc = extractDocFromBody(node)
 
-    # The result will be a temp block that holds the temporary variables.
-    result = StatementTempBlock(
-        source_ref = source_ref
+    # We need a scope for the temporary variables, and they might be closured.
+    temp_scope = provider.allocateTempScope(
+        name          = "class_creation",
+        allow_closure = True
     )
 
-    tmp_bases = result.getTempVariable( "bases" )
-    tmp_class_decl_dict = result.getTempVariable( "class_decl_dict" )
-    tmp_metaclass = result.getTempVariable( "metaclass" )
-    tmp_prepared = result.getTempVariable( "prepared" )
+    tmp_bases = provider.allocateTempVariable(
+        temp_scope = temp_scope,
+        name       = "bases"
+    )
+    tmp_class_decl_dict = provider.allocateTempVariable(
+        temp_scope = temp_scope,
+        name       = "class_decl_dict"
+    )
+    tmp_metaclass = provider.allocateTempVariable(
+        temp_scope = temp_scope,
+        name       = "metaclass"
+    )
+    tmp_prepared = provider.allocateTempVariable(
+        temp_scope = temp_scope,
+        name       = "prepared"
+    )
 
     class_creation_function = ExpressionFunctionBody(
         provider   = provider,
@@ -124,7 +133,8 @@ def _buildClassNode3( provider, node, source_ref ):
         source_ref = source_ref
     )
 
-    # Hack:
+    # Hack: This allows some APIs to work although this is not yet officially a
+    # child yet.
     class_creation_function.parent = provider
 
     body = buildStatementsNode(
@@ -134,7 +144,7 @@ def _buildClassNode3( provider, node, source_ref ):
         source_ref = source_ref
     )
 
-    # source_ref_orig = source_ref
+    source_ref_orig = source_ref
     source_ref = source_ref.atInternal()
 
     if body is not None:
@@ -144,7 +154,7 @@ def _buildClassNode3( provider, node, source_ref ):
     statements = [
         StatementSetLocals(
             new_locals = ExpressionTempVariableRef(
-                variable   = tmp_prepared.makeReference( result ),
+                variable   = tmp_prepared.makeReference(provider),
                 source_ref = source_ref
             ),
             source_ref = source_ref
@@ -159,7 +169,7 @@ def _buildClassNode3( provider, node, source_ref ):
                 source_ref    = source_ref,
                 user_provided = True
             ),
-            source_ref   = source_ref.atInternal()
+            source_ref   = source_ref
         )
     ]
 
@@ -179,11 +189,9 @@ def _buildClassNode3( provider, node, source_ref ):
             )
         )
 
+    # The "__qualname__" attribute is new in Python 3.3.
     if Utils.python_version >= 330:
-        if provider.isExpressionFunctionBody():
-            qualname = provider.getFunctionQualname() + ".<locals>." + node.name
-        else:
-            qualname = node.name
+        qualname = class_creation_function.getFunctionQualname()
 
         statements.append(
             StatementAssignmentVariable(
@@ -200,16 +208,37 @@ def _buildClassNode3( provider, node, source_ref ):
             )
         )
 
+    if Utils.python_version >= 340 and False: # TODO: Temporarily reverted:
+        tmp_class = class_creation_function.allocateTempVariable(
+            temp_scope = None,
+            name       = "__class__"
+        )
+
+        class_target_variable_ref = ExpressionTargetTempVariableRef(
+            variable   = tmp_class.makeReference(class_creation_function),
+            source_ref = source_ref
+        )
+        class_variable_ref = ExpressionTempVariableRef(
+            variable   = tmp_class.makeReference(class_creation_function),
+            source_ref = source_ref
+        )
+    else:
+        class_target_variable_ref = ExpressionTargetVariableRef(
+            variable_name = "__class__",
+            source_ref    = source_ref
+        )
+        class_variable_ref = ExpressionVariableRef(
+            variable_name = "__class__",
+            source_ref    = source_ref
+        )
+
     statements += [
         body,
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetVariableRef(
-                variable_name = "__class__",
-                source_ref    = source_ref
-            ),
+            variable_ref = class_target_variable_ref,
             source       = ExpressionCall(
                 called     = ExpressionTempVariableRef(
-                    variable   = tmp_metaclass.makeReference( result ),
+                    variable   = tmp_metaclass.makeReference(provider),
                     source_ref = source_ref
                 ),
                 args       = makeSequenceCreationOrConstant(
@@ -221,7 +250,7 @@ def _buildClassNode3( provider, node, source_ref ):
                             user_provided = True
                         ),
                         ExpressionTempVariableRef(
-                            variable   = tmp_bases.makeReference( result ),
+                            variable   = tmp_bases.makeReference(provider),
                             source_ref = source_ref
                         ),
                         ExpressionBuiltinLocals(
@@ -231,7 +260,7 @@ def _buildClassNode3( provider, node, source_ref ):
                     source_ref    = source_ref
                 ),
                 kw         = ExpressionTempVariableRef(
-                    variable   = tmp_class_decl_dict.makeReference( result ),
+                    variable   = tmp_class_decl_dict.makeReference(provider),
                     source_ref = source_ref
                 ),
                 source_ref = source_ref
@@ -239,10 +268,7 @@ def _buildClassNode3( provider, node, source_ref ):
             source_ref   = source_ref
         ),
         StatementReturn(
-            expression = ExpressionVariableRef(
-                variable_name = "__class__",
-                source_ref    = source_ref
-            ),
+            expression = class_variable_ref,
             source_ref = source_ref
         )
     ]
@@ -256,7 +282,7 @@ def _buildClassNode3( provider, node, source_ref ):
     # The class body is basically a function that implicitely, at the end
     # returns its locals and cannot have other return statements contained.
 
-    class_creation_function.setBody( body )
+    class_creation_function.setBody(body)
 
     # The class body is basically a function that implicitely, at the end
     # returns its created class and cannot have other return statements
@@ -279,22 +305,24 @@ def _buildClassNode3( provider, node, source_ref ):
 
     for decorator in buildNodeList(
             provider,
-            reversed( node.decorator_list ),
+            reversed(node.decorator_list),
             source_ref
         ):
         decorated_body = ExpressionCallNoKeywords(
             called     = decorator,
             args       = ExpressionMakeTuple(
-                elements   = ( decorated_body, ),
+                elements   = (
+                    decorated_body,
+                ),
                 source_ref = source_ref
             ),
             source_ref = decorator.getSourceReference()
         )
 
-    statements = [
+    statements = (
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_bases.makeReference( result ),
+                variable   = tmp_bases.makeReference( provider ),
                 source_ref = source_ref
             ),
             source       = makeSequenceCreationOrConstant(
@@ -308,7 +336,7 @@ def _buildClassNode3( provider, node, source_ref ):
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class_decl_dict.makeReference( result ),
+                variable   = tmp_class_decl_dict.makeReference(provider),
                 source_ref = source_ref
             ),
             source       = makeDictCreationOrConstant(
@@ -322,17 +350,18 @@ def _buildClassNode3( provider, node, source_ref ):
                     node.keywords
                 ],
                 values = [
-                    buildNode( provider, keyword.value, source_ref )
+                    buildNode(provider, keyword.value, source_ref)
                     for keyword in
                     node.keywords
                 ],
+                lazy_order = False,
                 source_ref = source_ref
             ),
             source_ref = source_ref
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_metaclass.makeReference( result ),
+                variable   = tmp_metaclass.makeReference(provider),
                 source_ref = source_ref
             ),
             source       = ExpressionSelectMetaclass(
@@ -346,7 +375,7 @@ def _buildClassNode3( provider, node, source_ref ):
                         ),
                         right      = ExpressionTempVariableRef(
                             variable   = tmp_class_decl_dict.makeReference(
-                                result
+                                provider
                             ),
                             source_ref = source_ref
                         ),
@@ -355,7 +384,7 @@ def _buildClassNode3( provider, node, source_ref ):
                     yes_expression = ExpressionDictOperationGet(
                         dicte      = ExpressionTempVariableRef(
                             variable   = tmp_class_decl_dict.makeReference(
-                                result
+                                provider
                             ),
                             source_ref = source_ref
                         ),
@@ -368,7 +397,7 @@ def _buildClassNode3( provider, node, source_ref ):
                     ),
                     no_expression  = ExpressionConditional(
                         condition      = ExpressionTempVariableRef(
-                            variable   = tmp_bases.makeReference( result ),
+                            variable   = tmp_bases.makeReference(provider),
                             source_ref = source_ref
                         ),
                         no_expression  = ExpressionBuiltinRef(
@@ -379,7 +408,7 @@ def _buildClassNode3( provider, node, source_ref ):
                             value      = ExpressionSubscriptLookup(
                                 expression = ExpressionTempVariableRef(
                                     variable   = tmp_bases.makeReference(
-                                        result
+                                        provider
                                     ),
                                     source_ref = source_ref
                                 ),
@@ -397,12 +426,12 @@ def _buildClassNode3( provider, node, source_ref ):
                     source_ref     = source_ref
                 ),
                 bases     = ExpressionTempVariableRef(
-                    variable   = tmp_bases.makeReference( result ),
+                    variable   = tmp_bases.makeReference(provider),
                     source_ref = source_ref
                 ),
                 source_ref = source_ref
             ),
-            source_ref = source_ref
+            source_ref = source_ref_orig
         ),
         StatementConditional(
             condition  = ExpressionComparison(
@@ -413,7 +442,9 @@ def _buildClassNode3( provider, node, source_ref ):
                     user_provided = True
                 ),
                 right      = ExpressionTempVariableRef(
-                    variable   = tmp_class_decl_dict.makeReference( result ),
+                    variable   = tmp_class_decl_dict.makeReference(
+                        provider
+                    ),
                     source_ref = source_ref
                 ),
                 source_ref = source_ref
@@ -423,7 +454,7 @@ def _buildClassNode3( provider, node, source_ref ):
                 statement = StatementDictOperationRemove(
                     dicte = ExpressionTempVariableRef(
                         variable   = tmp_class_decl_dict.makeReference(
-                            result
+                            provider
                         ),
                         source_ref = source_ref
                     ),
@@ -439,13 +470,17 @@ def _buildClassNode3( provider, node, source_ref ):
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_prepared.makeReference( result ),
+                variable   = tmp_prepared.makeReference(
+                    provider
+                ),
                 source_ref = source_ref
             ),
             source       = ExpressionConditional(
                 condition = ExpressionBuiltinHasattr(
                     object     = ExpressionTempVariableRef(
-                        variable   = tmp_metaclass.makeReference( result ),
+                        variable   = tmp_metaclass.makeReference(
+                            provider
+                        ),
                         source_ref = source_ref
                     ),
                     name       = ExpressionConstantRef(
@@ -463,7 +498,9 @@ def _buildClassNode3( provider, node, source_ref ):
                 yes_expression = ExpressionCall(
                     called     = ExpressionAttributeLookup(
                         expression     = ExpressionTempVariableRef(
-                            variable   = tmp_metaclass.makeReference( result ),
+                            variable   = tmp_metaclass.makeReference(
+                                provider
+                            ),
                             source_ref = source_ref
                         ),
                         attribute_name = "__prepare__",
@@ -477,7 +514,9 @@ def _buildClassNode3( provider, node, source_ref ):
                                 user_provided = True
                             ),
                             ExpressionTempVariableRef(
-                                variable   = tmp_bases.makeReference( result ),
+                                variable   = tmp_bases.makeReference(
+                                    provider
+                                ),
                                 source_ref = source_ref
                             )
                         ),
@@ -485,7 +524,7 @@ def _buildClassNode3( provider, node, source_ref ):
                     ),
                     kw         = ExpressionTempVariableRef(
                         variable   = tmp_class_decl_dict.makeReference(
-                            result
+                            provider
                         ),
                         source_ref = source_ref
                     ),
@@ -502,36 +541,58 @@ def _buildClassNode3( provider, node, source_ref ):
             ),
             source     = decorated_body,
             source_ref = source_ref
-        )
-    ]
+        ),
+    )
 
-    result.setBody(
-        StatementsSequence(
-            statements = statements,
+    final = (
+        StatementDelVariable(
+            variable_ref = ExpressionTargetTempVariableRef(
+                variable   = tmp_bases.makeReference(provider),
+                source_ref = source_ref
+            ),
+            tolerant   = True,
+            source_ref = source_ref
+        ),
+        StatementDelVariable(
+            variable_ref = ExpressionTargetTempVariableRef(
+                variable   = tmp_class_decl_dict.makeReference(provider),
+                source_ref = source_ref
+            ),
+            tolerant   = True,
+            source_ref = source_ref
+        ),
+        StatementDelVariable(
+            variable_ref = ExpressionTargetTempVariableRef(
+                variable   = tmp_metaclass.makeReference(provider),
+                source_ref = source_ref
+            ),
+            tolerant   = True,
+            source_ref = source_ref
+        ),
+        StatementDelVariable(
+            variable_ref = ExpressionTargetTempVariableRef(
+                variable   = tmp_prepared.makeReference(provider),
+                source_ref = source_ref
+            ),
+            tolerant   = True,
             source_ref = source_ref
         )
     )
 
-    return result
-
-
-def _buildClassNode2( provider, node, source_ref ):
-    class_statements, class_doc = extractDocFromBody( node )
-
-    # This function is the Python3 special case with special re-formulation as
-    # according to developer manual.
-
-    # The result will be a temp block that holds the temporary variables.
-    result = StatementTempBlock(
+    return makeTryFinallyStatement(
+        tried      = statements,
+        final      = final,
         source_ref = source_ref
     )
 
-    tmp_bases = result.getTempVariable( "bases" )
-    tmp_class_dict = result.getTempVariable( "class_dict" )
-    tmp_metaclass = result.getTempVariable( "metaclass" )
-    tmp_class = result.getTempVariable( "class" )
 
-    class_creation_function = ExpressionFunctionBody(
+def _buildClassNode2(provider, node, source_ref):
+    class_statements, class_doc = extractDocFromBody(node)
+
+    # This function is the Python2 special case with special re-formulation as
+    # according to developer manual.
+
+    function_body = ExpressionFunctionBody(
         provider   = provider,
         is_class   = True,
         parameters = make_class_parameters,
@@ -541,7 +602,7 @@ def _buildClassNode2( provider, node, source_ref ):
     )
 
     body = buildStatementsNode(
-        provider   = class_creation_function,
+        provider   = function_body,
         nodes      = class_statements,
         frame      = True,
         source_ref = source_ref
@@ -604,12 +665,19 @@ def _buildClassNode2( provider, node, source_ref ):
     # The class body is basically a function that implicitely, at the end
     # returns its locals and cannot have other return statements contained.
 
-    class_creation_function.setBody( body )
+    function_body.setBody( body )
+
+    temp_scope = provider.allocateTempScope( "class_creation" )
+
+    tmp_bases = provider.allocateTempVariable( temp_scope, "bases" )
+    tmp_class_dict = provider.allocateTempVariable(temp_scope, "class_dict")
+    tmp_metaclass = provider.allocateTempVariable(temp_scope, "metaclass")
+    tmp_class = provider.allocateTempVariable(temp_scope, "class")
 
     statements = [
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_bases.makeReference( result ),
+                variable   = tmp_bases.makeReference( provider ),
                 source_ref = source_ref
             ),
             source       = makeSequenceCreationOrConstant(
@@ -623,13 +691,13 @@ def _buildClassNode2( provider, node, source_ref ):
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class_dict.makeReference( result ),
+                variable   = tmp_class_dict.makeReference(provider),
                 source_ref = source_ref
             ),
             source       =   ExpressionFunctionCall(
                 function = ExpressionFunctionCreation(
                     function_ref = ExpressionFunctionRef(
-                        function_body = class_creation_function,
+                        function_body = function_body,
                         source_ref    = source_ref
                     ),
                     defaults     = (),
@@ -644,7 +712,7 @@ def _buildClassNode2( provider, node, source_ref ):
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_metaclass.makeReference( result ),
+                variable   = tmp_metaclass.makeReference(provider),
                 source_ref = source_ref
             ),
             source       = ExpressionConditional(
@@ -656,14 +724,14 @@ def _buildClassNode2( provider, node, source_ref ):
                         user_provided = True
                     ),
                     right      = ExpressionTempVariableRef(
-                        variable   = tmp_class_dict.makeReference( result ),
+                        variable   = tmp_class_dict.makeReference(provider),
                         source_ref = source_ref
                     ),
                     source_ref = source_ref
                 ),
                 yes_expression = ExpressionDictOperationGet(
                     dicte = ExpressionTempVariableRef(
-                        variable   = tmp_class_dict.makeReference( result ),
+                        variable   = tmp_class_dict.makeReference(provider),
                         source_ref = source_ref
                     ),
                     key   = ExpressionConstantRef(
@@ -676,7 +744,7 @@ def _buildClassNode2( provider, node, source_ref ):
                 no_expression = ExpressionSelectMetaclass(
                     metaclass = None,
                     bases     = ExpressionTempVariableRef(
-                        variable   = tmp_bases.makeReference( result ),
+                        variable   = tmp_bases.makeReference( provider ),
                         source_ref = source_ref
                     ),
                     source_ref = source_ref
@@ -687,12 +755,12 @@ def _buildClassNode2( provider, node, source_ref ):
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class.makeReference( result ),
+                variable   = tmp_class.makeReference(provider),
                 source_ref = source_ref
             ),
             source     = ExpressionCallNoKeywords(
                 called         = ExpressionTempVariableRef(
-                    variable   = tmp_metaclass.makeReference( result ),
+                    variable   = tmp_metaclass.makeReference(provider),
                     source_ref = source_ref
                 ),
                 args           = ExpressionMakeTuple(
@@ -703,11 +771,13 @@ def _buildClassNode2( provider, node, source_ref ):
                             user_provided = True
                         ),
                         ExpressionTempVariableRef(
-                            variable   = tmp_bases.makeReference( result ),
+                            variable   = tmp_bases.makeReference( provider ),
                             source_ref = source_ref
                         ),
                         ExpressionTempVariableRef(
-                            variable   = tmp_class_dict.makeReference( result ),
+                            variable   = tmp_class_dict.makeReference(
+                                provider
+                            ),
                             source_ref = source_ref
                         )
                     ),
@@ -716,7 +786,7 @@ def _buildClassNode2( provider, node, source_ref ):
                 source_ref = source_ref
             ),
             source_ref = source_ref
-        )
+        ),
     ]
 
     for decorator in buildNodeList(
@@ -727,7 +797,7 @@ def _buildClassNode2( provider, node, source_ref ):
         statements.append(
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_class.makeReference( result ),
+                    variable   = tmp_class.makeReference(provider),
                     source_ref = source_ref
                 ),
                 source       = ExpressionCallNoKeywords(
@@ -735,7 +805,9 @@ def _buildClassNode2( provider, node, source_ref ):
                     args       = ExpressionMakeTuple(
                         elements  = (
                             ExpressionTempVariableRef(
-                                variable   = tmp_class.makeReference( result ),
+                                variable   = tmp_class.makeReference(
+                                    provider
+                                ),
                                 source_ref = source_ref
                             ),
                         ),
@@ -754,29 +826,67 @@ def _buildClassNode2( provider, node, source_ref ):
                 source_ref    = source_ref
             ),
             source     = ExpressionTempVariableRef(
-                variable   = tmp_class.makeReference( result ),
+                variable   = tmp_class.makeReference(provider),
                 source_ref = source_ref
             ),
             source_ref = source_ref
         )
     )
 
-    result.setBody(
-        StatementsSequence(
-            statements = statements,
+    final = (
+        StatementDelVariable(
+            variable_ref = ExpressionTargetTempVariableRef(
+                variable   = tmp_class.makeReference(provider),
+                source_ref = source_ref
+            ),
+            tolerant   = True,
+            source_ref = source_ref
+        ),
+        StatementDelVariable(
+            variable_ref = ExpressionTargetTempVariableRef(
+                variable   = tmp_bases.makeReference( provider ),
+                source_ref = source_ref
+            ),
+            tolerant   = True,
+            source_ref = source_ref
+        ),
+        StatementDelVariable(
+            variable_ref = ExpressionTargetTempVariableRef(
+                variable   = tmp_class_dict.makeReference(provider),
+                source_ref = source_ref
+            ),
+            tolerant   = True,
+            source_ref = source_ref
+        ),
+        StatementDelVariable(
+            variable_ref = ExpressionTargetTempVariableRef(
+                variable   = tmp_metaclass.makeReference(provider),
+                source_ref = source_ref
+            ),
+            tolerant   = True,
             source_ref = source_ref
         )
     )
 
-    return result
+    return makeTryFinallyStatement(
+        tried      = statements,
+        final      = final,
+        source_ref = source_ref
+    )
 
-def buildClassNode( provider, node, source_ref ):
+
+def buildClassNode(provider, node, source_ref):
     assert getKind( node ) == "ClassDef"
 
     # Python2 and Python3 are similar, but fundamentally different, so handle
     # them in dedicated code.
 
-    if Utils.python_version >= 300:
-        return _buildClassNode3( provider, node, source_ref )
-    else:
-        return _buildClassNode2( provider, node, source_ref )
+    pushIndicatorVariable(Ellipsis)
+
+    try:
+        if Utils.python_version >= 300:
+            return _buildClassNode3(provider, node, source_ref)
+        else:
+            return _buildClassNode2(provider, node, source_ref)
+    finally:
+        popIndicatorVariable()
