@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -15,57 +15,69 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
+""" Reformulation of lambda expressions.
+
+Consult the developmer manual for information. TODO: Add ability to sync
+source code comments with developer manual sections.
+
+"""
 
 from nuitka import Utils
-
-from nuitka.nodes.VariableRefNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTempVariableRef,
-    StatementTempBlock
-)
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
-from nuitka.nodes.FunctionNodes import (
-    ExpressionFunctionCreation,
-    ExpressionFunctionBody,
-    ExpressionFunctionRef
-)
-from nuitka.nodes.StatementNodes import (
-    StatementExpressionOnly,
-    StatementsSequence,
-    StatementsFrame
+from nuitka.nodes.AssignNodes import (
+    StatementAssignmentVariable,
+    StatementDelVariable
 )
 from nuitka.nodes.ComparisonNodes import ExpressionComparisonIsNOT
 from nuitka.nodes.ConditionalNodes import StatementConditional
-from nuitka.nodes.YieldNodes import ExpressionYield
+from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.FunctionNodes import (
+    ExpressionFunctionBody,
+    ExpressionFunctionCreation,
+    ExpressionFunctionRef
+)
 from nuitka.nodes.ReturnNodes import StatementReturn
-from nuitka.nodes.AssignNodes import StatementAssignmentVariable
+from nuitka.nodes.StatementNodes import StatementExpressionOnly, StatementsFrame
+from nuitka.nodes.VariableRefNodes import (
+    ExpressionTargetTempVariableRef,
+    ExpressionTempVariableRef
+)
+from nuitka.nodes.YieldNodes import ExpressionYield
 
+from .Helpers import (
+    buildNode,
+    buildNodeList,
+    getKind,
+    makeStatementsSequenceFromStatement,
+    makeTryFinallyStatement,
+    mergeStatements
+)
 from .ReformulationFunctionStatements import (
     buildParameterAnnotations,
     buildParameterKwDefaults,
     buildParameterSpec
 )
 
-from .Helpers import (
-    makeStatementsSequenceFromStatement,
-    buildNodeList,
-    buildNode,
-    getKind
-)
 
-def buildLambdaNode( provider, node, source_ref ):
+def buildLambdaNode(provider, node, source_ref):
     assert getKind( node ) == "Lambda"
 
+    parameters = buildParameterSpec( "<lambda>", node, source_ref )
+
     function_body = ExpressionFunctionBody(
         provider   = provider,
         name       = "<lambda>",
         doc        = None,
-        parameters = buildParameterSpec( "<lambda>", node, source_ref ),
+        parameters = parameters,
         source_ref = source_ref,
     )
 
-    defaults = buildNodeList( provider, node.args.defaults, source_ref )
-    kw_defaults = buildParameterKwDefaults( provider, node, function_body, source_ref )
+    defaults = buildNodeList(provider, node.args.defaults, source_ref)
+    kw_defaults = buildParameterKwDefaults(
+        provider      = provider,
+        node          = node,
+        function_body = function_body,
+        source_ref    = source_ref
+    )
 
     body = buildNode(
         provider   = function_body,
@@ -75,16 +87,17 @@ def buildLambdaNode( provider, node, source_ref ):
 
     if function_body.isGenerator():
         if Utils.python_version < 270:
-            temp_block = StatementTempBlock(
-                source_ref = source_ref,
+            tmp_return_value = function_body.allocateTempVariable(
+                temp_scope = None,
+                name       = "yield_return"
             )
 
-            tmp_return_value = temp_block.getTempVariable( "yield_return" )
-
             statements = (
                 StatementAssignmentVariable(
                     variable_ref = ExpressionTargetTempVariableRef(
-                        variable = tmp_return_value.makeReference( temp_block ),
+                        variable = tmp_return_value.makeReference(
+                            function_body
+                        ),
                         source_ref = source_ref,
                     ),
                     source     = body,
@@ -93,7 +106,9 @@ def buildLambdaNode( provider, node, source_ref ):
                 StatementConditional(
                     condition = ExpressionComparisonIsNOT(
                         left       = ExpressionTempVariableRef(
-                            variable = tmp_return_value.makeReference( temp_block ),
+                            variable = tmp_return_value.makeReference(
+                                function_body
+                            ),
                             source_ref = source_ref,
                         ),
                         right      = ExpressionConstantRef(
@@ -106,7 +121,9 @@ def buildLambdaNode( provider, node, source_ref ):
                         statement = StatementExpressionOnly(
                             expression = ExpressionYield(
                                 expression = ExpressionTempVariableRef(
-                                    variable = tmp_return_value.makeReference( temp_block ),
+                                    variable = tmp_return_value.makeReference(
+                                        function_body
+                                    ),
                                     source_ref = source_ref,
                                 ),
                                 source_ref = source_ref
@@ -118,15 +135,20 @@ def buildLambdaNode( provider, node, source_ref ):
                     source_ref = source_ref
                 )
             )
-
-            temp_block.setBody(
-                StatementsSequence(
-                    statements = statements,
+            body = makeTryFinallyStatement(
+                tried      = statements,
+                final      = StatementDelVariable(
+                    variable_ref = ExpressionTargetTempVariableRef(
+                        variable = tmp_return_value.makeReference(
+                            function_body
+                        ),
+                        source_ref = source_ref,
+                    ),
+                    tolerant   = True,
                     source_ref = source_ref
-                )
+                ),
+                source_ref = source_ref
             )
-
-            body = temp_block
         else:
             body = StatementExpressionOnly(
                 expression = body,
@@ -139,17 +161,22 @@ def buildLambdaNode( provider, node, source_ref ):
         )
 
     body = StatementsFrame(
-        statements    = ( body, ),
+        statements    = mergeStatements(
+            (body,)
+        ),
         guard_mode    = "generator" if function_body.isGenerator() else "full",
-        arg_names     = function_body.getParameters().getCoArgNames(),
-        kw_only_count = function_body.getParameters().getKwOnlyParameterCount(),
+        var_names     = parameters.getCoArgNames(),
+        arg_count     = parameters.getArgumentCount(),
+        kw_only_count = parameters.getKwOnlyParameterCount(),
+        has_starlist  = parameters.getStarListArgumentName() is not None,
+        has_stardict  = parameters.getStarDictArgumentName() is not None,
         code_name     = "<lambda>",
         source_ref    = body.getSourceReference()
     )
 
-    function_body.setBody( body )
+    function_body.setBody(body)
 
-    annotations = buildParameterAnnotations( provider, node, source_ref )
+    annotations = buildParameterAnnotations(provider, node, source_ref)
 
     return ExpressionFunctionCreation(
         function_ref = ExpressionFunctionRef(
