@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -15,15 +15,21 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
+""" Reformulation of yield and yield from expressions.
 
-from nuitka.nodes.YieldNodes import ExpressionYield, ExpressionYieldFrom
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+Consult the developmer manual for information. TODO: Add ability to sync
+source code comments with developer manual sections.
+
+"""
+
+from nuitka import SyntaxErrors, Utils
 from nuitka.nodes.BuiltinIteratorNodes import ExpressionBuiltinIter1
-
-from nuitka import Utils, SyntaxErrors
+from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.YieldNodes import ExpressionYield, ExpressionYieldFrom
 
 from .Helpers import buildNode
 
+
 def _markAsGenerator(provider, node, source_ref):
     if provider.isPythonModule():
         SyntaxErrors.raiseSyntaxError(
@@ -53,6 +59,7 @@ def buildYieldNode(provider, node, source_ref):
             source_ref = source_ref
         )
 
+
 def buildYieldFromNode(provider, node, source_ref):
     assert Utils.python_version >= 330
 
