@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -19,10 +19,10 @@
 import tempfile, sys, os
 
 print "eval 3+3=", eval("3+3")
-print "eval  3+3=", eval(" 3+3")
+print "eval  4+4=", eval(" 4+4")
 
 def functionEval1():
-    return eval(" 3+3")
+    return eval(" 5+5")
 
 print "eval in a function with nothing provided", functionEval1()
 
@@ -98,7 +98,8 @@ def functionExecfile():
     local_vars = dict( global_vars )
 
     print "execfile with globals and locals dict in a function:",
-    print execfile( tmp_filename, global_vars, local_vars ),
+    x = execfile( tmp_filename, global_vars, local_vars )
+    print x,
     print global_vars.keys(), local_vars, e, f
 
 functionExecfile()
