@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -16,15 +16,21 @@
 #     limitations under the License.
 #
 
-print u"gfcrk"
-print repr( u"g\xfcrk" )
+print(u"gfcrk")
+print(repr(u"g\xfcrk"))
 
-print r"""\x00"""
+print(r"""\x00""")
 
-print "\ttest\n"
+print("\ttest\n")
 
-print """
+print("""
 something
 with
 new
-lines"""
+lines""")
+
+# TODO: Python3 has a problem here, hard to find, disabled for now.
+if False:
+    encoding = "utf-16-be"
+    print("[\uDC80]".encode(encoding))
+    print("[\\udc80]")
